"""Scanners that evaluate dental practice compliance documentation and checklists."""

from __future__ import annotations

from .practice_scanner import PracticeScanner
from .checklist_scanner import ChecklistScanner

SCANNERS = {
    "practice": PracticeScanner,
    "checklist": ChecklistScanner,
}

__all__ = ["PracticeScanner", "ChecklistScanner", "SCANNERS"]
